;===========================================================================================
; TimeWin.asm  Fasm Flat assembler
; Temps de travail de Windows
; Programm par AsmGges France
;===========================================================================================
; NOTES:
; Hauteur fentre static pour texte = mini 22 (pour compatibilit texte avec WIN95)
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

  WSCREEN = 280
  HSCREEN = 62

;===========================================================================================
; Dbut du code programme
;-------------------------------------------------------------------------------------------
section '.code' code readable executable

start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne
    ; ----------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur
    ; -----------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Valider le choix de la structure WNDCLASS
    ; -----------------------------------------
    invoke RegisterClass,wc

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,WSCREEN
    shr eax,1
    mov [rect.left],eax

    ; Ajuster les coordonnes de la fentre
    ; -------------------------------------
    mov [rect.top],0
    mov [rect.right],WSCREEN
    mov [rect.bottom],HSCREEN
    invoke AdjustWindowRectEx,rect,WS_CAPTION+WS_VISIBLE+WS_SYSMENU+WS_MINIMIZEBOX,\
                                                                            0,NULL
    mov eax,[rect.bottom]
    sub eax,3                ;hauteur bord infrieur fentre active
    sub eax,[rect.top]
    sub eax,HSCREEN
    inc eax                  ;hauteur bord infrieur bandeau titre
    mov [HBarTitle],eax      ;hauteur totale du bandeau titre

    invoke GetSystemMetrics,SM_CYSCREEN
    sub eax,HSCREEN
    sub eax,[HBarTitle]
    shr eax,1
    mov [rect.top],100       ;forage de Y sinon remplacer la valeur par eax

    mov eax,HSCREEN
    add eax,[HBarTitle]

    ; Cration fentre principale avec barre de titre suprieure
    ; ----------------------------------------------------------
    invoke CreateWindowEx,NULL,ClassName,WinTitreFr,\
                          WS_CAPTION+WS_VISIBLE+WS_SYSMENU+WS_MINIMIZEBOX,\
                          [rect.left],[rect.top],WSCREEN+6,eax,NULL,0,[wc.hInstance],NULL
    mov [hWin],eax

    ; Affichage de la fentre
    ; -----------------------
    invoke ShowWindow,[hWin],SW_SHOWDEFAULT
    ; Rafraichissement de la fentre
    ; ------------------------------
    invoke UpdateWindow,[hWin]

 ; Boucle de traitement des messages
 ; ---------------------------------
 MessageLoop:
    invoke GetMessage,msg,0,0,0
    or eax,eax
    jz ExitProgram

    invoke TranslateMessage,msg
    invoke DispatchMessage,msg

    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        ; Dtruire le timer
        ; -----------------
        invoke KillTimer,[@hWnd],1000

        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgPub,ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande la fin de l'application
        ; -------------------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE
        ; Cration fonte
        ; --------------
        invoke CreateFont,8,0,0,0,400,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                          CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,NFont
        mov [hFont],eax

        ; Groupbox AsmGges
        ; ----------------
        stdcall Ctl,Button,ClassName,[@hWnd],8,2,264,20,-1,BS_GROUPBOX,0
        ; Affecter la fonte au control
        ; ----------------------------
        invoke SendMessage,eax,WM_SETFONT,[hFont],NULL

        ; Cration d'une fentre static pour l'affichage du temps
        ; -------------------------------------------------------
        stdcall Ctl,Static,NoTxt,[@hWnd],8,32,264,22,-1,SS_CENTER,WS_EX_CLIENTEDGE
        mov [hStatic],eax
        ; Affecter la fonte au control
        ; ----------------------------
        invoke SendMessage,[hStatic],WM_SETFONT,[hFont],NULL

        ; Initialiser un timer
        ; --------------------
        invoke SetTimer,[@hWnd],1000,100,NULL

    ; Calculs heures/minutes/secondes
    ; -------------------------------
    .elseif [@uMsg] = WM_TIMER
        invoke GetTickCount
        mov edx,0  
        mov ecx,3600000 
        div ecx 
        mov [Heures],eax
        mov eax,edx
        mov edx,0 
        mov ecx,60000  
        div ecx
        mov [Minutes],eax
        mov eax,edx 
        mov edx,0 
        mov ecx,1000 
        div ecx 
        mov [Secondes],eax

        ; Calcul des heures
        ; -----------------
        .if [Heures] < 10
            ; Convertion valeur dans une chaine
            ; ---------------------------------
            cinvoke wsprintf,HeuresBuf,sFormat,[Heures]
         
            mov edi,NoTxt
            inc edi
            mov esi,HeuresBuf
            lodsb 
            stosb   
            mov esi,NoTxt
            mov edi,HeuresBuf
            mov ecx,2 
            rep movsb 

        .else 
            cinvoke wvsprintf,HeuresBuf,sFormat,[Heures]
            
        .endif

        ; Calcul des minutes
        ; ------------------
        .if [Minutes] < 10
            cinvoke wsprintf,MinutesBuf,sFormat,[Minutes]
            
            mov edi,NoTxt
            inc edi
            mov esi,MinutesBuf
            lodsb
            stosb
            mov esi,NoTxt
            mov edi,MinutesBuf
            mov ecx,2
            rep movsb
            
        .else
            cinvoke wsprintf,MinutesBuf,sFormat,[Minutes]
            
        .endif

        ; Calcul des secondes
        ; -------------------
        .if [Secondes] < 10
            cinvoke wsprintf,SecondesBuf,sFormat,[Secondes]
            
            mov edi,NoTxt
            inc edi
            mov esi,SecondesBuf
            lodsb
            stosb
            mov esi,NoTxt
            mov edi,SecondesBuf
            mov ecx,2
            rep movsb
            
        .else
            cinvoke wsprintf,SecondesBuf,sFormat,[Secondes]
            
        .endif

        ; Concat des chanes
        ; ------------------
        cinvoke wsprintf,ResultBuf,ExpTxtFr,HeuresBuf,MinutesBuf,SecondesBuf

        ; Texte complet dans la fentre statique
        ; --------------------------------------
        invoke SetWindowText,[hStatic],ResultBuf

    .elseif [@uMsg] = WM_PAINT
        ; Dbut criture/dessin
        ; ---------------------
        invoke BeginPaint,[@hWnd],ps

        ; Fin criture/dessin
        ; -------------------
        invoke EndPaint,[@hWnd],ps

    .else
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret

endp

;===========================================================================================
; Cration d'une fentre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class   = adresse du nom de control class
; @AsrText = adresse du control text
; @hParent = handle de la fentre parent
; @X       = x-coordonne fentre-enfant
; @Y       = y-coordonne fentre-enfant
; @wX      = largeur en pixels fentre-enfant
; @hY      = hauteur en pixels fentre-enfant
; @Id      = identificateur du control forme ID_XXX ou -1 si pas de ID
; @Style   = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle = window style tendu
;-------------------------------------------------------------------------------------------
; sortie eax = handle de la fentre-enfant
;-------------------------------------------------------------------------------------------
proc Ctl @Class, @AdrText, @hParent, @X, @Y, @wX, @hY, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AdrText],eax,[@X],[@Y],[@wX],[@hY],\
                          [@hParent],[@Id],[wc.hInstance],0
    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  ClassName    db ' AsmGges Win32',0
  WinTitre     db ' Time and Windows  ',0
  WinTitreFr   db ' Temps et Windows  ',0
  MsgPub       db ' AsmGges @2006 France',0

  sFormat      db '%u',0
  Static       db 'STATIC',0
  Button       db 'BUTTON',0
  NFont        db 'Ms Sans Serif',0
  ExpTxt       db ' Windows works since:  %s:%s:%s',0
  ExpTxtFr     db ' Windows travaille depuis:  %sh %smn %ss',0
  NoTxt        db '0',0,0
  align 4

  hWin         dd ?
  hFont        dd ?
  HBarTitle    dd ?
  hStatic      dd ?

  Heures       dd ?
  Minutes      dd ?
  Secondes     dd ?

  ResultBuf    rb 100
  HeuresBuf    rb 10
  align 4
  MinutesBuf   rb 10
  align 4
  SecondesBuf  rb 10
  align 4

  wc           WNDCLASS  0,WindowProc,0,0,0,0,0,COLOR_BTNFACE+1,0,ClassName
  msg          MSG
  ps           PAINTSTRUCT
  rect         RECT

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons

  IDI_FRANCE = 17

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

;===========================================================================================
